package com.b3.rbl.compensation.controller;

import java.time.Duration;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseCookie;
import org.springframework.http.ResponseEntity;

import lombok.extern.log4j.Log4j2;

@Log4j2
public abstract class BaseController {
    protected HttpServletRequest request;
    protected HttpServletResponse response;

    protected BaseController(HttpServletRequest request, HttpServletResponse response) {
        this.request = request;
        this.response = response;
    }

    public void sendRedirect(String url) {
        response.setHeader("Location", url);
        response.setStatus(302);
    }

    protected ResponseEntity<?> notAuthenticated() {
        HttpHeaders httpHeaders = new HttpHeaders();
        return new ResponseEntity<>(httpHeaders, HttpStatus.UNAUTHORIZED);
    }

    public ResponseCookie generateCookie(String name, String value) {
        ResponseCookie cookie = ResponseCookie.from(name, value) // key & value
                .httpOnly(true).secure(true).maxAge(Duration.ofHours(1)).sameSite("None") // sameSite
                .build();
        return cookie;
    }

    protected Map<String, String> getParameter() {
        Map<String, String> params = new HashMap<String, String>();
        for (Enumeration<String> e = this.request.getParameterNames(); e.hasMoreElements();) {
            String name = e.nextElement();
            String value = this.request.getParameter(name);
            log.info("params: [{},{}]", name, value);
            params.put(name, value);
        }
        return params;

    }

}
