package com.b3.rbl.compensation.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;

import com.b3.rbl.compensation.db.entity.RblSchedule;
import com.b3.rbl.compensation.db.repository.RblScheduleRepository;
import com.b3.rbl.compensation.model.GenerateReportRequestModel;
import com.b3.rbl.compensation.service.DatainExcelService;
import com.b3.rbl.compensation.service.OnedriveService;
import com.b3.rbl.compensation.service.ReportService;
import com.b3.rbl.compensation.service.VariableService;

import lombok.extern.log4j.Log4j2;

import org.springframework.web.bind.annotation.PostMapping;

@Log4j2
@Controller
@RequestMapping(value = "/generate-report")
public class GenerateReportController extends BaseController {
    private ReportService reportService;
    private DatainExcelService datainExcelService;
    private OnedriveService onedriveService;
    private RblScheduleRepository scheduleRepository;
    private VariableService variableService;

    public GenerateReportController(HttpServletRequest request, HttpServletResponse response,
            RblScheduleRepository scheduleRepository, ReportService reportService,
            DatainExcelService datainExcelService, OnedriveService onedriveService, VariableService variableService) {
        super(request, response);
        this.reportService = reportService;
        this.datainExcelService = datainExcelService;
        this.onedriveService = onedriveService;
        this.scheduleRepository = scheduleRepository;
        this.variableService = variableService;
    }

    @GetMapping
    public String report(ModelMap model) {
        model.addAttribute("scheduleList", reportService.getScheduleMaster());
        model.addAttribute("schedule", new GenerateReportRequestModel());
        model.addAttribute("error", "");
        return "pages/generate-reports/generate-report";
    }

    @PostMapping
    public String generateReport(ModelMap model, @ModelAttribute GenerateReportRequestModel schedule) {
        String error = "";
        model.addAttribute("scheduleList", reportService.getScheduleMaster());
        model.addAttribute("schedule", schedule);
        log.info(schedule);
        String process = variableService.findByKey(VariableService.Generate_Report_Process.TYPE_NAME);
        if (VariableService.Generate_Report_Process.RUNNING.equals(process)) {
            model.addAttribute("error", "Generate report process is running now! Please try again.");
            return "pages/generate-reports/generate-report";
        } else {
            variableService.updateValue(VariableService.Generate_Report_Process.TYPE_NAME,
                    VariableService.Generate_Report_Process.RUNNING);
        }

        if (schedule.getScheduleId() != null) {
            RblSchedule scheduleEntity = this.scheduleRepository.findById(schedule.getScheduleId()).get();
            if (!schedule.getGenerateFinancialReport() && !schedule.getUploadStatement()) {
                error = "Please select processes type";
            } else {
                try {
                    if (schedule.getGenerateFinancialReport()) {
                        datainExcelService.readDatain(scheduleEntity);
                        error = "Generate Report Successfull.";
                    }
                    if (schedule.getUploadStatement()) {
                        onedriveService.getFileFromPath(scheduleEntity.getPeriodYear(), scheduleEntity.getPeriodMonth(),
                                scheduleEntity.getCycleName());
                        error = "Upload Statement/Hold Report Successfull.";
                    }

                } catch (Exception e) {
                    error = e.getMessage();
                    e.printStackTrace();
                }
            }
        } else {
            error = "Please select perfomance year month";
        }
        model.addAttribute("error", error);
        variableService.updateValue(VariableService.Generate_Report_Process.TYPE_NAME, null);
        return "pages/generate-reports/generate-report";
    }
}
