package com.b3.rbl.compensation.controller;

import java.util.ArrayList;
import java.util.Base64;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.MapUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.CookieValue;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import com.b3.rbl.compensation.db.entity.Account;
import com.b3.rbl.compensation.service.AccountService;
import com.b3.rbl.compensation.service.JwtUtil;
import com.b3.rbl.compensation.service.ReportService;
import com.b3.rbl.compensation.util.ObjectUtil;

import lombok.extern.log4j.Log4j2;

@Log4j2
@Controller
@RequestMapping(value = "/report")
public class ReportController extends BaseController {

    private ReportService reportService;
    private AccountService accountService;
    private JwtUtil jwtUtil;

    public ReportController(JwtUtil jwtUtil, HttpServletRequest request, HttpServletResponse response,
            ReportService reportService, AccountService accountService) {
        super(request, response);
        this.jwtUtil = jwtUtil;
        this.reportService = reportService;
        this.accountService = accountService;
    }

    @GetMapping
    public String report(ModelMap model, @RequestParam(value = "token", required = false) String token,
            @RequestParam(value = "uid", required = false) String uid) throws Exception {
        String tokenUuid = null;
        if (ObjectUtil.stringIsNull(uid)) {
            return "error/401";
        }

        if (!ObjectUtil.stringIsNull(token)) {
            tokenUuid = this.jwtUtil.extractUuid(token);
        }

        if (tokenUuid == null || !uid.equals(tokenUuid)) {

            Account account = null;
            if ((account = accountService.getAccount(uid)) == null) {
                return "error/401";
            }
            Map<String, Object> info = Collections.singletonMap("agentCode", account.getAgentCode());
            token = this.jwtUtil.generateToken(info, account.getId());

        }
        model.addAttribute("scheduleList", reportService.getScheduleMaster());
        model.addAttribute("reportMaster", reportService.getReportMaster());
        model.addAttribute("reportList", new ArrayList());
        model.addAttribute("token", token);
        model.addAttribute("uid", uid);

        return "pages/reports/report-list";
    }

    @GetMapping("search")
    public String searchReport(ModelMap model, @RequestParam(name = "uid") String uid,
            @RequestParam(value = "token", required = false) String token,
            @RequestParam(name = "periodId", required = false, defaultValue = "") String periodId,
            @RequestParam(name = "reportName") String reportName,
            @RequestParam(name = "agentCode", required = false) String agentCode,
            @RequestParam(name = "action", required = false) String action) throws Exception {

        if (ObjectUtil.stringIsNull(token) || ObjectUtil.stringIsNull(uid)) {
            return "error/401";
        }
        String myAgentCode = jwtUtil.extractClaim(token, c -> c.get("agentCode", String.class));
        log.info(myAgentCode);

        List rs = reportService.searchReport(reportName, periodId, myAgentCode, agentCode);
        model.addAttribute("reportList", rs);
        model.addAttribute("scheduleList", reportService.getScheduleMaster());
        model.addAttribute("reportMaster", reportService.getReportMaster());
        model.addAllAttributes(getParameter());
        return "pages/reports/report-list";
    }

}
