package com.b3.rbl.compensation.db;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;
import java.util.stream.Collectors;

import javax.sql.DataSource;
import com.b3.rbl.compensation.util.ObjectUtil;
import javax.persistence.EntityManager;
import javax.transaction.Transactional;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
public class MetadataExtractor {
    private final DataSource datasource;
    @Autowired
    private EntityManager apiDatasource;
    private String schema;

    public MetadataExtractor(DataSource datasource,
            @Value("${spring.jpa.properties.hibernate.default_schema}") String schema) throws SQLException {
        this.datasource = datasource;
        this.schema = schema;
        System.out.println("schema " + schema);
        extractTableInfo();
        // extractColumnInfo("paid_detail");
    }

    public List<String> extractTableInfo() throws SQLException {
        DatabaseMetaData databaseMetaData = datasource.getConnection().getMetaData();
        List<String> x = new ArrayList<>();
        try (ResultSet resultSet = databaseMetaData.getTables(null, schema, null, new String[] { "TABLE" })) {
            while (resultSet.next()) {
                log.info("Table Name[{}] Remark: {}", resultSet.getString("TABLE_NAME"),
                        resultSet.getString("REMARKS"));
                x.add(resultSet.getString("TABLE_NAME"));
            }
        }
        return x;
    }

    public void extractSystemTables() throws SQLException {
        DatabaseMetaData databaseMetaData = datasource.getConnection().getMetaData();
        try (ResultSet resultSet = databaseMetaData.getTables(null, null, null, new String[] { "SYSTEM TABLE" })) {
            while (resultSet.next()) {
                // Print the names of system tables
                System.out.println(resultSet.getString("TABLE_NAME"));
            }
        }
    }

    public void extractViews() throws SQLException {
        DatabaseMetaData databaseMetaData = datasource.getConnection().getMetaData();
        try (ResultSet resultSet = databaseMetaData.getTables(null, null, null, new String[] { "VIEW" })) {
            while (resultSet.next()) {
                // Print the names of existing views
                System.out.println(resultSet.getString("TABLE_NAME"));
            }
        }
    }

    public void extractColumnInfo(String tableName, String fileData) throws SQLException {
        DatabaseMetaData databaseMetaData = datasource.getConnection().getMetaData();
        System.out.println("File String ::: " + fileData);
        List<Map<String, String>> dataColumn1 = new ArrayList<Map<String, String>>();
        try (ResultSet columns = databaseMetaData.getColumns(null, null, tableName, null)) {
            while (columns.next()) {
                Map<String, String> dataColumn = new HashMap<>();
                // String columnName = columns.getString("COLUMN_NAME");
                // String columnSize = columns.getString("COLUMN_SIZE");
                // String datatype = columns.getString("DATA_TYPE");
                // String isNullable = columns.getString("IS_NULLABLE");
                // String isAutoIncrement = columns.getString("IS_AUTOINCREMENT");

                dataColumn.put("columnName", columns.getString("COLUMN_NAME"));
                dataColumn.put("columnSize", columns.getString("COLUMN_SIZE"));
                dataColumn.put("datatype", columns.getString("DATA_TYPE"));
                dataColumn.put("isNullable", columns.getString("IS_NULLABLE"));
                dataColumn.put("isAutoIncrement", columns.getString("IS_AUTOINCREMENT"));
                // System.out.println("dataColumn>>> "+dataColumn);

                dataColumn1.add(dataColumn);
                // System.out.println(String.format(
                // "ColumnName: %s, columnSize: %s, datatype: %s, isColumnNullable: %s,
                // isAutoIncrementEnabled: %s",
                // columnName, columnSize, datatype, isNullable, isAutoIncrement));
            }
            // System.out.println("dataColumn>>> " + dataColumn1);
            System.out.println("dataColumnMapping" + getColumnMapping());
        }

        // return dataColumn1;
    }

    public void extractPrimaryKeys(String tableName) throws SQLException {
        DatabaseMetaData databaseMetaData = datasource.getConnection().getMetaData();
        try (ResultSet primaryKeys = databaseMetaData.getPrimaryKeys(null, null, tableName)) {
            while (primaryKeys.next()) {
                String primaryKeyColumnName = primaryKeys.getString("COLUMN_NAME");
                String primaryKeyName = primaryKeys.getString("PK_NAME");
                System.out.println(String.format("columnName:%s, pkName:%s", primaryKeyColumnName, primaryKeyName));
            }
        }
    }

    public void extractForeignKeys(String tableName) throws SQLException {
        DatabaseMetaData databaseMetaData = datasource.getConnection().getMetaData();
        try (ResultSet foreignKeys = databaseMetaData.getImportedKeys(null, null, tableName)) {
            while (foreignKeys.next()) {
                String pkTableName = foreignKeys.getString("PKTABLE_NAME");
                String fkTableName = foreignKeys.getString("FKTABLE_NAME");
                String pkColumnName = foreignKeys.getString("PKCOLUMN_NAME");
                String fkColumnName = foreignKeys.getString("FKCOLUMN_NAME");
                System.out.println(String.format("pkTableName:%s, fkTableName:%s, pkColumnName:%s, fkColumnName:%s",
                        pkTableName, fkTableName, pkColumnName, fkColumnName));
            }
        }
    }

    public void extractDatabaseInfo() throws SQLException {
        DatabaseMetaData databaseMetaData = datasource.getConnection().getMetaData();
        String productName = databaseMetaData.getDatabaseProductName();
        String productVersion = databaseMetaData.getDatabaseProductVersion();

        String driverName = databaseMetaData.getDriverName();
        String driverVersion = databaseMetaData.getDriverVersion();

        System.out.println(String.format("Product name:%s, Product version:%s", productName, productVersion));
        System.out.println(String.format("Driver name:%s, Driver Version:%s", driverName, driverVersion));
    }

    public void extractUserName() throws SQLException {
        DatabaseMetaData databaseMetaData = datasource.getConnection().getMetaData();
        String userName = databaseMetaData.getUserName();
        System.out.println(userName);
        try (ResultSet schemas = databaseMetaData.getSchemas()) {
            while (schemas.next()) {
                String table_schem = schemas.getString("TABLE_SCHEM");
                String table_catalog = schemas.getString("TABLE_CATALOG");
                System.out.println(String.format("Table_schema:%s, Table_catalog:%s", table_schem, table_catalog));
            }
        }
    }

    public void extractSupportedFeatures() throws SQLException {
        DatabaseMetaData databaseMetaData = datasource.getConnection().getMetaData();
        System.out.println("Supports scrollable & Updatable Result Set: " + databaseMetaData
                .supportsResultSetConcurrency(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE));
        System.out.println("Supports Full Outer Joins: " + databaseMetaData.supportsFullOuterJoins());
        System.out.println("Supports Stored Procedures: " + databaseMetaData.supportsStoredProcedures());
        System.out.println("Supports Subqueries in 'EXISTS': " + databaseMetaData.supportsSubqueriesInExists());
        System.out.println("Supports Transactions: " + databaseMetaData.supportsTransactions());
        System.out.println("Supports Core SQL Grammar: " + databaseMetaData.supportsCoreSQLGrammar());
        System.out.println("Supports Batch Updates: " + databaseMetaData.supportsBatchUpdates());
        System.out.println("Supports Column Aliasing: " + databaseMetaData.supportsColumnAliasing());
        System.out.println("Supports Savepoints: " + databaseMetaData.supportsSavepoints());
        System.out.println("Supports Union All: " + databaseMetaData.supportsUnionAll());
        System.out.println("Supports Union: " + databaseMetaData.supportsUnion());
    }

    public List<Map<String, Object>> getColumnMapping() {

        String sql = "SELECT x.* FROM compensation.table_mapping x ORDER BY Id ASC ;";

        List<Map<String, Object>> data = null;
        try {
            org.hibernate.query.Query query = this.apiDatasource.createNativeQuery(sql)
                    .unwrap(org.hibernate.query.Query.class)
                    .setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);

            data = ObjectUtil.columnToCamelCase(query.getResultList());

        } catch (Exception e) {

        }
        return data;
    }

    @Transactional
    public String insertDataUpload(String tableName, String fileData) {
        String message;
        String sql = "INSERT INTO compensation.";
        sql = sql + tableName + "(";
        List<Map<String, Object>> columnMapping = getColumnMapping();
        String[] fileDataArr = fileData.split("\n");
        String[] columnDataArr = fileDataArr[0].split(",");
        List<String> columnList = Arrays.asList(columnDataArr);
        List<String> typeList = new ArrayList<>();

        List<Map<String, Object>> columnTable = columnMapping.stream()
                .filter(res -> res.containsValue(tableName))
                .collect(Collectors.toList());

        for (int i = 0; i < columnDataArr.length; i++) {

            String columnFind = columnList.get(i);
            // List<Map<String, Object>> columnTable = columnMapping.stream()
            // .filter(res -> res.containsValue(columnFind) && res.containsValue(tableName))
            // .collect(Collectors.toList());

            typeList.add(columnTable.get(i).get("tableType").toString());
            sql = sql + "\"" + columnTable.get(i).get("tableColumn") + "\"";
            if (i < columnDataArr.length - 1) {
                sql = sql + ",";
            } else {
                sql = sql + ") VALUES";
            }
        }
        for (int i = 1; i < fileDataArr.length; i++) {
            String[] valuesDataArr = fileDataArr[i].split(",", -1);
            List<String> valuesList = Arrays.asList(valuesDataArr);
            sql = sql + valuesSQL(typeList, valuesList, columnTable.size());
            if (i < fileDataArr.length - 1) {
                sql = sql + ",";
            } else {
                sql = sql + ";";
            }
        }
        System.out.println("SQL >>> " + sql);

        try {
            this.apiDatasource.createNativeQuery(sql)
                    .unwrap(org.hibernate.query.Query.class)
                    .setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP).executeUpdate();
            // query.executeUpdate();
            // data = ObjectUtil.columnToCamelCase(query);
            message = "อัปโหลดข้อมูลสำเร็จ";
        } catch (Exception e) {
            e.printStackTrace();
            message = "อัปโหลดข้อมูลไม่สำเร็จกรุณาตรวจสอบชื่อ ตาราง และไฟล์";
        }

        return message;

    }

    public String valuesSQL(List<String> typeList, List<String> valuesSql, int size) {
        String valuesString = "(";

        for (int i = 0; i < size; i++) {

            if (typeList.get(i).equals("String")) {
                valuesString = valuesString + "'" + valuesSql.get(i) + "'";
            } else if (typeList.get(i).equals("Number")) {
                valuesString = valuesString + (valuesSql.get(i).isEmpty() ? 0 : valuesSql.get(i));
            } else if (typeList.get(i).equals("Date")) {
                valuesString = valuesString + (valuesSql.get(i).isEmpty() ? null : "'" + valuesSql.get(i) + "'");
            }

            if (i < size - 1) {
                valuesString = valuesString + ",";
            }

        }
        valuesString = valuesString + ")";
        return valuesString;
    }

}