package com.b3.rbl.compensation.rest;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.modelmapper.ModelMapper;
import org.modelmapper.convention.MatchingStrategies;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;

import com.b3.rbl.compensation.db.entity.Account;
import com.b3.rbl.compensation.db.entity.AgBenefit;
import com.b3.rbl.compensation.db.entity.AgCustomer;
import com.b3.rbl.compensation.db.entity.AgFinancialTransaction;
import com.b3.rbl.compensation.db.entity.AgPolicyMaster;
import com.b3.rbl.compensation.model.salesforce.AccountDto;
import com.b3.rbl.compensation.model.salesforce.AgBenefitDto;
import com.b3.rbl.compensation.model.salesforce.AgCustomerDto;
import com.b3.rbl.compensation.model.salesforce.AgFinancialTransactionDto;
import com.b3.rbl.compensation.model.salesforce.AgPolicyMasterDto;
import com.b3.rbl.compensation.model.salesforce.SfdcObject;
import com.b3.rbl.compensation.service.DatainService;

import lombok.extern.log4j.Log4j2;

import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

@Log4j2
@RestController
@RequestMapping("datain")
public class DataInRestApi {

    private final ModelMapper modelMapper;
    private DatainService datainService;

    public DataInRestApi(DatainService datainService) {
        this.datainService = datainService;
        this.modelMapper = new ModelMapper();
        modelMapper.getConfiguration().setMatchingStrategy(MatchingStrategies.STRICT);
    }

    @PostMapping("upsert-account")
    public ResponseEntity<DatainService.WebserviceRespone> upsertAccount(
            @RequestBody List<SfdcObject<AccountDto>> datain) {
        log.info(datain);
        DatainService.WebserviceRespone res = null;
        List<Account> accountList = new ArrayList<>();
        try {
            int i = 0;
            for (SfdcObject<AccountDto> d : datain) {
                log.info("{}. Data {} ", ++i, d);
                log.info("----------------------------------");
                Account acc = modelMapper.map(d.getData(), Account.class);
                log.info(acc);
                accountList.add(acc);
            }
            res = datainService.upsertAccount(accountList);
        } catch (Exception ex) {
            log.error(ex);
        }
        return ResponseEntity.ok().body(res);
    }

    @PostMapping("upsert-customer")
    public ResponseEntity<DatainService.WebserviceRespone> upsertCustomer(
            @RequestBody List<SfdcObject<AgCustomerDto>> datain) {
        log.info(datain);
        DatainService.WebserviceRespone res = null;
        List<AgCustomer> customerList = new ArrayList<>();
        try {
            int i = 0;
            for (SfdcObject<AgCustomerDto> d : datain) {
                log.info("{}. Data {} ", ++i, d);
                log.info("----------------------------------");
                AgCustomer customer = modelMapper.map(d.getData(), AgCustomer.class);
                log.info(customer);
                customerList.add(customer);
            }
            res = datainService.upsertCustomer(customerList);
        } catch (Exception ex) {
            log.error(ex);
        }
        return ResponseEntity.ok().body(res);
    }

    @PostMapping("upsert-financial")
    public ResponseEntity<DatainService.WebserviceRespone> upsertFinancial(
            @RequestBody List<SfdcObject<AgFinancialTransactionDto>> datain) {
        log.info(datain);
        DatainService.WebserviceRespone res = null;
        List<AgFinancialTransaction> financialList = new ArrayList<>();
        try {
            int i = 0;
            for (SfdcObject<AgFinancialTransactionDto> d : datain) {
                log.info("{}. Data {} ", ++i, d);
                log.info("----------------------------------");
                log.info(d.getData());
                AgFinancialTransaction finance = modelMapper.map(d.getData(), AgFinancialTransaction.class);
                log.info(finance);
                financialList.add(finance);
            }
            res = datainService.upsertFinancial(financialList);
        } catch (Exception ex) {
            log.error(ex);
        }
        return ResponseEntity.ok().body(res);
    }

    @PostMapping("upsert-policy")
    public ResponseEntity<DatainService.WebserviceRespone> upsertPolicy(
            @RequestBody List<SfdcObject<AgPolicyMasterDto>> datain) {
        log.info(datain);
        DatainService.WebserviceRespone res = null;
        List<AgPolicyMaster> policyList = new ArrayList<>();
        try {
            int i = 0;
            for (SfdcObject<AgPolicyMasterDto> d : datain) {
                log.info("{}. Data {} ", ++i, d);
                log.info("----------------------------------");
                AgPolicyMaster policy = modelMapper.map(d.getData(), AgPolicyMaster.class);
                log.info(policy);
                policyList.add(policy);
            }
            res = datainService.upsertPolicy(policyList);
        } catch (Exception ex) {
            log.error(ex);
        }
        return ResponseEntity.ok().body(res);
    }

    @PostMapping("upsert-benefit")
    public ResponseEntity<DatainService.WebserviceRespone> upsertBenefit(
            @RequestBody List<SfdcObject<AgBenefitDto>> datain) {
        log.info(datain);
        DatainService.WebserviceRespone res = null;
        List<AgBenefit> benefitList = new ArrayList<>();
        try {
            int i = 0;
            for (SfdcObject<AgBenefitDto> d : datain) {
                log.info("{}. Data {} ", ++i, d);
                log.info("----------------------------------");
                AgBenefit benefit = modelMapper.map(d.getData(), AgBenefit.class);
                log.info(benefit);
                benefitList.add(benefit);
            }
            res = datainService.upsertBenefit(benefitList);
        } catch (Exception ex) {
            log.error(ex);
        }
        return ResponseEntity.ok().body(res);
    }

}
