package com.b3.rbl.compensation.rest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.b3.rbl.compensation.controller.BaseController;
import com.b3.rbl.compensation.service.JwtUtil;
import com.b3.rbl.compensation.service.OnedriveService;
import com.b3.rbl.onedrive.client.RequestTool;

@RestController
@RequestMapping("report-api")
public class ReportRestApi extends BaseController {

    private static final String ROOT_DRIVE = "/me/drive/root:/";
    public static final String PATH = RequestTool.BASE_URL + ROOT_DRIVE;
    private OnedriveService onedriveService;
    private JwtUtil jwtUtil;

    @Value("${report.onedrive.path.statement.prefix}")
    private String reportPathStatementPrefix;

    public ReportRestApi(OnedriveService onedriveService, JwtUtil jwtUtil, HttpServletRequest request,
            HttpServletResponse response) {
        super(request, response);
        this.onedriveService = onedriveService;
        this.jwtUtil = jwtUtil;
    }

    @GetMapping("/download")
    public ResponseEntity<byte[]> downloadReport(@RequestParam("downloadPath") String downloadPath,
            @RequestParam("token") String token, @RequestParam("filename") String filename,
            @RequestParam("mimeType") String mimeType) throws Exception {

        if (jwtUtil.isTokenExpired(token)) {
            super.response.setStatus(HttpServletResponse.SC_UNAUTHORIZED); // HTTP 401.
            return null;
        }

        String path = PATH + reportPathStatementPrefix + "/" + downloadPath.replace(" ", "%20") + ":/content";
        byte[] report = onedriveService.getFileFromUrl(path);
        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Disposition", "inline; filename=" + filename);
        headers.set(HttpHeaders.CONTENT_TYPE, mimeType + ";charset=utf-8");

        return ResponseEntity.ok().headers(headers).body(report);
    }
}
