package com.b3.rbl.compensation.service;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import javax.persistence.EntityManager;

import org.apache.commons.collections4.map.HashedMap;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Scope;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.b3.rbl.compensation.db.entity.RblSchedule;
import com.b3.rbl.compensation.db.entity.TmpAccount;
import com.b3.rbl.compensation.db.entity.TmpDebtCompany;
import com.b3.rbl.compensation.db.entity.TmpPayAll;
import com.b3.rbl.compensation.db.entity.TmpPayCompany;
import com.b3.rbl.compensation.db.entity.TmpPayIndividual;
import com.b3.rbl.compensation.db.entity.TmpSummary;
import com.b3.rbl.compensation.db.repository.RblScheduleRepository;
import com.b3.rbl.compensation.db.repository.TmpAccountRepository;
import com.b3.rbl.compensation.db.repository.TmpDeptCompanyRepository;
import com.b3.rbl.compensation.db.repository.TmpPayAllRepository;
import com.b3.rbl.compensation.db.repository.TmpPayCompanyRepository;
import com.b3.rbl.compensation.db.repository.TmpPayIndividualRepository;
import com.b3.rbl.compensation.db.repository.TmpSummaryRepository;
import com.b3.rbl.compensation.report.JasperService;
import com.b3.rbl.compensation.util.ObjectUtil;
import com.b3.rbl.onedrive.client.RequestTool;
import com.b3.rbl.onedrive.exceptions.ErrorResponseException;

import lombok.extern.log4j.Log4j2;
import net.sf.jasperreports.engine.JRException;

@Log4j2
@Service
public class DatainExcelService {

    private static final String SHEET_ACCOUNT_NAME = "Account";
    private static final String SHEET_SUMMARY_NAME = "Summary";
    private static final String SHEET_PAY_ALL_NAME = "Pay_All";
    private static final String SHEET_PAY_INDI_NAME = "Pay (Individual)";
    private static final String SHEET_PAY_COMP_NAME = "Pay (Company)";
    private static final String SHEET_DEPT_COMP_NAME = "Debt";
    private static final String[] SHEET_ACCOUNT_HEADER = new String[] { "docNo", "company", "doctype", "postingDate",
            "documentDate", "exchangeRate", "currency", "reference", "docHeaderText", "branch", "lineItem", "pk",
            "specialGl", "glAcc", "customer", "amount", "amountInLocalCurThb", "costCenter", "profitCenter",
            "internalOrder", "businessArea", "businessPlace", "taxAmount", "vatCode", "assignment", "lineItemText",
            "ref1", "ref2", "ref3", "whtType", "whtCode", "whtBase", "profitCenter", "channel", "prdCode", "rider",
            "level7", "sector", "checkingAmount", "code" };

    private static final String[] SHEET_PAID_ALL_HEADER = new String[] { "no", "agentCode", "name", "agentLevel",
            "smUnitName", "smUnitCode", "smName", "sdName", "esdName", "agentOffice", "incomeTax", "amountNonTax",
            "incomeAmountNonTax", "deductAmountNonTax", "amountFund", "incomeFund", "deductFund", "tax", "netIncome",
            "bankCode", "bankBranch", "bankName", "bankAccount", "fyc", "ryc", "recruiting", "faRetention", "faCareer",
            "faBonusM", "faBonusQ", "faBonusY", "motivation", "unitOverridingFy", "unitOverridingRy", "unitJoining",
            "unitFixAllow", "unitActiveFa", "unitRetention", "unitCareer", "unitExpandTier1", "unitExpandTier2",
            "unitExpandExtra", "unitExpandGroup", "unitBonusQ", "unitBonusY", "groupOverridingFy", "groupOverridingRy",
            "groupJoining", "groupBuilding", "groupFixAllow", "groupRetention", "groupCareer", "groupExpandTier1",
            "groupExpandTier2", "groupBonusQ", "groupBonusY", "lineOverriding", "lineFixAllow", "lineExpandTier1",
            "lineBonusQ", "lineBonusY", "officeFirstYear", "officeRenewalYear", "debt", "debtMotivation", "debtFundCom",
            "debtFundUnit", "debtFundGroup", "debtCreditFee", "debtMedical", "debtOther", "debtShortagePremium",
            "debtTranferCash", "debtTranferDebt", "debtReciveDebt", "title", "firstname", "lastname", "idcard",
            "postcode" };

    private static final String[] SHEET_SUMMARY_HEADER = new String[] { "code", "description", "includeTax",
            "withoutTax", "releaseHold", "total", "hold", "grandTotal" };
    private static final String[] SHEET_PAID_INDI_HEADER = new String[] { "no", "agentCode", "name", "agentLevel",
            "smUnitName", "smUnitCode", "smName", "sdName", "esdName", "agentOffice", "incomeTax", "amountNonTax",
            "incomeAmountNonTax", "deductAmountNonTax", "amountFund", "incomeFund", "deductFund", "tax", "netIncome",
            "bankCode", "bankBranch", "bankName", "bankAccount", "fyc", "ryc", "recruiting", "faRetention", "faCareer",
            "faBonusM", "faBonusQ", "faBonusY", "motivation", "unitOverridingFy", "unitOverridingRy", "unitJoining",
            "unitFixAllow", "unitActiveFa", "unitRetention", "unitCareer", "unitExpandTier1", "unitExpandTier2",
            "unitExpandExtra", "unitExpandGroup", "unitBonusQ", "unitBonusY", "groupOverridingFy", "groupOverridingRy",
            "groupJoining", "groupBuilding", "groupFixAllow", "groupRetention", "groupCareer", "groupExpandTier1",
            "groupExpandTier2", "groupBonusQ", "groupBonusY", "lineOverriding", "lineFixAllow", "lineExpandTier1",
            "lineBonusQ", "lineBonusY", "officeFirstYear", "officeRenewalYear", "debt", "debtMotivation", "debtFundCom",
            "debtFundUnit", "debtFundGroup", "debtCreditFee", "debtMedical", "debtOther", "debtShortagePremium",
            "debtTranferCash", "debtTranferDebt", "debtReciveDebt" };

    private static final String[] SHEET_PAID_COMP_HEADER = new String[] { "no", "agentCode", "name", "agentLevel",
            "smUnitName", "smUnitCode", "smName", "sdName", "esdName", "agentOffice", "incomeTax", "amountNonTax",
            "incomeAmountNonTax", "deductAmountNonTax", "amountFund", "incomeFund", "deductFund", "tax", "netIncome",
            "bankCode", "bankBranch", "bankName", "bankAccount", "fyc", "ryc", "recruiting", "faRetention", "faCareer",
            "faBonusM", "faBonusQ", "faBonusY", "motivation", "unitOverridingFy", "unitOverridingRy", "unitJoining",
            "unitFixAllow", "unitActiveFa", "unitRetention", "unitCareer", "unitExpandTier1", "unitExpandTier2",
            "unitExpandExtra", "unitExpandGroup", "unitBonusQ", "unitBonusY", "groupOverridingFy", "groupOverridingRy",
            "groupJoining", "groupFixAllow", "groupRetention", "groupCareer", "groupExpandTier1", "groupExpandTier2",
            "groupBonusQ", "groupBonusY", "lineOverriding", "lineFixAllow", "lineExpandTier1", "lineBonusQ",
            "lineBonusY", "officeFirstYear", "officeRenewalYear", "debt", "debtMotivation", "debtFundCom",
            "debtFundUnit", "debtFundGroup", "debtCreditFee", "debtMedical", "debtOther", "debtShortagePremium",
            "debtTranferCash", "debtTranferDebt", "debtReciveDebt" };

    private static final String[] SHEET_DEPT_COMP_HEADER = new String[] { "no", "agentCode", "name", "agentLevel",
            "smUnitName", "smUnitCode", "smName", "sdName", "esdName", "agentOffice", "incomeTax", "amountNonTax",
            "incomeAmountNonTax", "deductAmountNonTax", "amountFund", "incomeFund", "deductFund", "tax", "netIncome",
            "bankCode", "bankBranch", "bankName", "bankAccount", "fyc", "ryc", "recruiting", "faRetention", "faCareer",
            "faBonusM", "faBonusQ", "faBonusY", "motivation", "unitOverridingFy", "unitOverridingRy", "unitJoining",
            "unitFixAllow", "unitActiveFa", "unitRetention", "unitCareer", "unitExpandTier1", "unitExpandTier2",
            "unitExpandExtra", "unitExpandGroup", "unitBonusQ", "unitBonusY", "groupOverridingFy", "groupOverridingRy",
            "groupJoining", "groupFixAllow", "groupRetention", "groupCareer", "groupExpandTier1", "groupExpandTier2",
            "groupBonusQ", "groupBonusY", "lineOverriding", "lineFixAllow", "lineExpandTier1", "lineBonusQ",
            "lineBonusY", "officeFirstYear", "officeRenewalYear", "debt", "debtMotivation", "debtFundCom",
            "debtFundUnit", "debtFundGroup", "debtCreditFee", "debtMedical", "debtOther", "debtShortagePremium",
            "debtTranferCash", "debtTranferDebt", "debtReciveDebt" };

    private static final String ROOT_DRIVE = "/me/drive/root:";
    @Value("${report.onedrive.path.agent.payment:PROD_AgentcyPayment}")
    private String agentPaymentPath;
    private static final String PATH = RequestTool.BASE_URL + ROOT_DRIVE;

    private static final DateFormat SDF_YYYYMMDDHHMM = new SimpleDateFormat("yyyyMMddHHmm", new Locale("th", "TH"));
    private static final DateFormat SDF_YYYYMMDD = new SimpleDateFormat("yyyyMMdd");
    private static final DateFormat SDF_DD_MMMM_YYYY = new SimpleDateFormat("dd MMMM yyyy", new Locale("th", "TH"));
    private TmpAccountRepository tmpAccountRepository;
    private OnedriveService onedriveService;
    private TmpSummaryRepository tmpSummaryRepository;
    private TmpPayAllRepository tmpPayAllRepository;
    private TmpPayIndividualRepository tmpPayIndividualRepository;
    private TmpPayCompanyRepository tmpPayCompanyRepository;
    private TmpDeptCompanyRepository tmpDeptCompanyRepository;
    private JasperService jasperService;
    private RblScheduleRepository scheduleRepository;
    private EntityManager em;

    public DatainExcelService(TmpAccountRepository tmpAccountRepository, OnedriveService onedriveService,
            TmpSummaryRepository tmpSummaryRepository, TmpPayAllRepository tmpPayAllRepository,
            TmpPayIndividualRepository tmpPayIndividualRepository, TmpPayCompanyRepository tmpPayCompanyRepository,
            TmpDeptCompanyRepository tmpDeptCompanyRepository, RblScheduleRepository scheduleRepository,
            EntityManager em, JasperService jasperService) {
        this.tmpAccountRepository = tmpAccountRepository;
        this.onedriveService = onedriveService;
        this.tmpSummaryRepository = tmpSummaryRepository;
        this.tmpPayAllRepository = tmpPayAllRepository;
        this.tmpPayIndividualRepository = tmpPayIndividualRepository;
        this.tmpPayCompanyRepository = tmpPayCompanyRepository;
        this.tmpDeptCompanyRepository = tmpDeptCompanyRepository;
        this.scheduleRepository = scheduleRepository;
        this.em = em;
        this.jasperService = jasperService;

    }

    @Modifying
    @Transactional
    public List<TmpPayIndividual> readDataTmpPayIndividual(String paidDate, Sheet sheet) {
        List<TmpPayIndividual> list = new ArrayList<>();

        if (sheet == null) {
            log.info("ReadDataTmpPayIndividual sheet.getLastRowNum() {}", sheet.getLastRowNum());
            return list;
        }

        this.tmpPayIndividualRepository.deleteByPaidDate(new Integer(paidDate));
        log.info("ReadDataTmpPayAll sheet.getLastRowNum() {}", sheet.getLastRowNum());
        for (int i = 5; i < sheet.getLastRowNum() - 1; i++) {
            Row row = sheet.getRow(i);
            Map<String, Object> r = new HashMap<>();
            r.put("paidDate", paidDate);

            for (int c = 0; c < 75; c++) {
                Cell cell = row.getCell(c);
                Object o = getCellValue(cell);

                // log.info("[{},{}]", i, c);
                // log.info("[{},{}] : {}", row.getRowNum(), cell.getColumnIndex(), o);
                try {
                    String header = SHEET_PAID_INDI_HEADER[cell.getColumnIndex()];
                    r.put(header, o);
                } catch (ArrayIndexOutOfBoundsException | NullPointerException e) {
                    log.warn("Error at Row[{},{}] : {} ", row.getRowNum(), c, o);
                    break;
                }
            }
            // log.info(r);

            TmpPayIndividual tmp = ObjectUtil.mapper.convertValue(r, TmpPayIndividual.class);
            list.add(tmp);
        }
        this.tmpPayIndividualRepository.saveAll(list);
        return list;
    }

    @Modifying
    @Transactional
    public List<TmpPayCompany> readDataTmpPayCompany(String paidDate, Sheet sheet) {
        List<TmpPayCompany> list = new ArrayList<>();

        if (sheet == null) {
            log.info("Read Data TmpPayCompany sheet.getLastRowNum() {}", sheet.getLastRowNum());
            return list;
        }

        this.tmpPayCompanyRepository.deleteByPaidDate(new Integer(paidDate));
        log.info("ReadDataTmpPayCompany sheet.getLastRowNum() {}", sheet.getLastRowNum());
        for (int i = 5; i < sheet.getLastRowNum() - 1; i++) {
            Row row = sheet.getRow(i);
            Map<String, Object> r = new HashMap<>();
            r.put("paidDate", paidDate);

            for (int c = 0; c < 74; c++) {
                Cell cell = row.getCell(c);
                Object o = getCellValue(cell);

                // log.info("[{},{}]", i, c);
                // log.info("[{},{}] : {}", row.getRowNum(), cell.getColumnIndex(), o);
                try {
                    String header = SHEET_PAID_COMP_HEADER[cell.getColumnIndex()];
                    r.put(header, o);
                } catch (ArrayIndexOutOfBoundsException | NullPointerException e) {
                    log.warn("Error at Row[{}] : {} ", row.getRowNum(), o);
                    break;
                }
            }
            log.info(r);

            TmpPayCompany tmp = ObjectUtil.mapper.convertValue(r, TmpPayCompany.class);
            list.add(tmp);
        }
        this.tmpPayCompanyRepository.saveAll(list);
        return list;
    }

    @Modifying
    @Transactional
    public List<TmpDebtCompany> readDataTmpDeptCompany(String paidDate, Sheet sheet) {
        List<TmpDebtCompany> list = new ArrayList<>();

        if (sheet == null) {
            log.info("Read Data TmpDebtCompany sheet.getLastRowNum() {}", sheet.getLastRowNum());
            return list;
        }

        this.tmpDeptCompanyRepository.deleteByPaidDate(new Integer(paidDate));
        log.info("Read Data TmpDebtCompany sheet.getLastRowNum() {}", sheet.getLastRowNum());
        for (int i = 5; i < sheet.getLastRowNum(); i++) {
            Row row = sheet.getRow(i);
            Map<String, Object> r = new HashMap<>();
            r.put("paidDate", paidDate);

            for (int c = 0; c < 74; c++) {
                Cell cell = row.getCell(c);
                Object o = getCellValue(cell);

                // log.info("[{},{}]", i, c);
                // log.info("[{},{}] : {}", row.getRowNum(), cell.getColumnIndex(), o);
                try {
                    String header = SHEET_DEPT_COMP_HEADER[cell.getColumnIndex()];
                    r.put(header, o);
                } catch (ArrayIndexOutOfBoundsException | NullPointerException e) {
                    log.warn("Error at Row[{}] : {} ", row.getRowNum(), o);
                    break;
                }
            }
            log.info(r);

            TmpDebtCompany tmp = ObjectUtil.mapper.convertValue(r, TmpDebtCompany.class);
            list.add(tmp);
        }
        this.tmpDeptCompanyRepository.saveAll(list);
        return list;
    }

    @Transactional
    public void readDatain(RblSchedule schedule) throws Exception {
        String paidDate = SDF_YYYYMMDD.format(schedule.getPaymentDate());
        readDatain(String.format("%s%s", schedule.getPeriodYear(), schedule.getPeriodMonth()), paidDate, schedule);

    }

    public Path generateDirectReport(String filename, String paidDate, String min, String max) throws IOException {
        String dt = SDF_YYYYMMDDHHMM.format(new Date());
        String sql = "select * from compensation.F_REPORT_DIRECT_CREDIT (:paidDate, :reportDate, :min, :max)";
        List<String> rs = em.createNativeQuery(sql).setParameter("paidDate", new Integer(paidDate))
                .setParameter("reportDate", dt).setParameter("min", new java.math.BigDecimal(min))
                .setParameter("max", new java.math.BigDecimal(max)).getResultList();

        return buildReport(filename + dt + ".txt", rs);
    }

    public Path generateCoverDirectReport(String paidDate) throws IOException {
        String sql = "select  * from compensation.F_REPORT_cover_DIRECT_CREDIT (:paidDate)";
        List<String> rs = em.createNativeQuery(sql).setParameter("paidDate", new Integer(paidDate)).getResultList();
        String dt = SDF_YYYYMMDDHHMM.format(new Date());
        return buildReport("DirectPayLog" + dt + ".txt", rs);
    }

    public Path generatePndReport(String period) throws IOException {
        String sql = "select * from compensation.f_report_pnd (:period)";
        List<String> rs = em.createNativeQuery(sql).setParameter("period", new Integer(period)).getResultList();

        return buildReport("PND1.txt", rs);
    }

    public enum HrmsReportType {
        COMPANY("Company"), INDIVIDUAL("Individual"), DEBT("Debt");

        private String name;

        HrmsReportType(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

    }

    public Path generateHrmsReport(HrmsReportType reportType, String paidDate, String periodFromTo, String reportDate)
            throws IOException, JRException, SQLException {
        Integer groupSize = getHrmsCountGroupSize(reportType.getName().toUpperCase(), paidDate);
        byte[] data = this.jasperService.generateBRMSPdfReport(reportType.getName().toUpperCase(), paidDate,
                periodFromTo, groupSize);
        return buildReport("HRMS " + reportType.getName() + reportDate + ".pdf", data);
    }

    public Path generateSmartReport(String filename, String paidDate, String min, String max) throws IOException {
        String sql = "select  * from compensation.F_REPORT_SMART_CREDIT(:paidDate, :min, :max)";
        List<String> rs = em.createNativeQuery(sql).setParameter("paidDate", new Integer(paidDate))
                .setParameter("min", new java.math.BigDecimal(min)).setParameter("max", new java.math.BigDecimal(max))
                .getResultList();
        String dt = SDF_YYYYMMDDHHMM.format(new Date());
        return buildReport(filename + dt + ".txt", rs);
    }

    public Path generateCoverSmartReport(String paidDate) throws IOException {
        String sql = "select  * from compensation.F_REPORT_Cover_SMART_CREDIT(:paidDate)";
        List<String> rs = em.createNativeQuery(sql).setParameter("paidDate", new Integer(paidDate)).getResultList();
        String dt = SDF_YYYYMMDDHHMM.format(new Date());
        return buildReport("SmartPayLog" + dt + ".txt", rs);
    }

    public Path buildReport(String fileName, List<String> line) throws IOException {
        Path path = Files.createTempFile(null, "-" + fileName);
        Files.write(path, line, StandardCharsets.UTF_8);
        return path;
    }

    public Path buildReport(String fileName, byte[] data) throws IOException {
        Path path = Files.createTempFile(null, "-" + fileName);
        Files.write(path, data);
        return path;
    }

    @Modifying
    @Transactional
    public void readDatain(String schedule, String paidDate, RblSchedule scheduleEntity) throws Exception {
        String url = String.format("%s/%s/%s/import/%s_AgencyPayment_%s.xlsx:/content", PATH, agentPaymentPath,
                schedule, schedule, paidDate);
        String exportPath = String.format("/%s/%s/export/%s", agentPaymentPath, schedule, paidDate);

        log.info("URL : {}", url);
        byte[] excle = onedriveService.getFileFromUrl(url);

        if (excle == null) {
            throw new Exception("New file " + url + " not found.");
        }
        try (Workbook workbook = new XSSFWorkbook(new ByteArrayInputStream(excle))) {

            Date start = new Date();

            // log.info("Start execute ReadDataAccount @ {}", start);
            // Sheet sheetAccount = workbook.getSheet(SHEET_ACCOUNT_NAME);
            // List<TmpAccount> listAcc = readDataAccount(paidDate, sheetAccount);
            // log.info("End execute ReadDataAccount @ {}, use time: {}", new Date(),
            // System.currentTimeMillis() - start.getTime());

            // start = new Date();
            // log.info("Start execute ReadDataTmpSummary @ {}", start);
            // Sheet sheetSummary = workbook.getSheet(SHEET_SUMMARY_NAME);
            // List<TmpSummary> listSummaries = readDataTmpSummary(paidDate, sheetSummary);
            // log.info("End ReadDataTmpSummary ReadDataAccount @ {}, use time: {}", new
            // Date(),
            // System.currentTimeMillis() - start.getTime());

            start = new Date();
            log.info("Start execute ReadDataTmpPayAll @ {}", start);
            Sheet sheetPayAllSheet = workbook.getSheet(SHEET_PAY_ALL_NAME);
            List<TmpPayAll> listpayAll = readDataTmpPayAll(paidDate, sheetPayAllSheet);
            log.info("total records: {}", listpayAll.size());
            log.info("End ReadDataTmpPayAll ReadDataAccount @ {}, use time: {}", new Date(),
                    System.currentTimeMillis() - start.getTime());

            start = new Date();
            log.info("Start execute generate DirectReport @ {}", start);
            this.directReportProcess(exportPath, paidDate);
            log.info("End execute generate DirectReport @ @ {}, use time: {}", new Date(),
                    System.currentTimeMillis() - start.getTime());

            start = new Date();
            log.info("Start execute generate CoverDirectReport @ {}", start);
            this.coverDirectReportProcess(exportPath, paidDate);
            log.info("End execute generate CoverDirectReport @ @ {}, use time: {}", new Date(),
                    System.currentTimeMillis() - start.getTime());

            start = new Date();
            log.info("Start execute generate SmartReport @ {}", start);
            this.smartReportProcess(exportPath, paidDate);
            log.info("End execute generate SmartReport @ @ {}, use time: {}", new Date(),
                    System.currentTimeMillis() - start.getTime());

            start = new Date();
            log.info("Start execute generate CoverSmartReport @ {}", start);
            this.coverSmartReportProcess(exportPath, paidDate);
            log.info("End execute generate CoverSmartReport @ @ {}, use time: {}", new Date(),
                    System.currentTimeMillis() - start.getTime());

            start = new Date();
            log.info("Start execute generate PNDReport @ {}", start);
            this.pndReportProcess(exportPath, schedule);
            log.info("End execute generate PNDReport @ @ {}, use time: {}", new Date(),
                    System.currentTimeMillis() - start.getTime());

            start = new Date();
            log.info("Start execute generate HRMSReport @ {}", start);
            hrmsReportProcess(exportPath, scheduleEntity);
            log.info("End execute generate HRMSReport @ @ {}, use time: {}", new Date(),
                    System.currentTimeMillis() - start.getTime());

            // start = new Date();
            // log.info("Start execute ReadDataTmpPayIndividual @ {}", start);
            // Sheet sheetPayIndiSheet = workbook.getSheet(SHEET_PAY_INDI_NAME);
            // List<TmpPayIndividual> listPayIndi = readDataTmpPayIndividual(paidDate,
            // sheetPayIndiSheet);
            // log.info("End ReadDataTmpPayIndividual @ {}, use time: {}", new Date(),
            // System.currentTimeMillis() - start.getTime());

            // start = new Date();
            // log.info("Start execute ReadDataTmpPayCompany @ {}", start);
            // Sheet sheetPayComp = workbook.getSheet(SHEET_PAY_COMP_NAME);
            // List<TmpPayCompany> listPayComp = readDataTmpPayCompany(paidDate,
            // sheetPayComp);
            // log.info("End ReadDataTmpPayCompany @ {}, use time: {}", new Date(),
            // System.currentTimeMillis() - start.getTime());

            // start = new Date();
            // log.info("Start execute Read Data TmpDeptCompany @ {}", start);
            // Sheet sheetDeptComp = workbook.getSheet(SHEET_DEPT_COMP_NAME);
            // List<TmpDebtCompany> listDeptComp = readDataTmpDeptCompany(paidDate,
            // sheetDeptComp);
            // log.info("End Read Data TmpDeptCompany @ {}, use time: {}", new Date(),
            // System.currentTimeMillis() - start.getTime());

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public void directReportProcess(String exportPath, String paidDate) throws IOException, ErrorResponseException {
        String[][] params = { { "DirectPayOut", "0", "1000000" }, { "2DirectPayOut", "1000000", "99999999999" }, };

        for (String[] param : params) {
            Path directPath = generateDirectReport(param[0], paidDate, param[1], param[2]);
            String directFileName = directPath.getFileName().toString().split("-")[1];
            onedriveService.saveFile(exportPath, directFileName, directPath);
            directPath.toFile().deleteOnExit();
        }
    }

    public void coverDirectReportProcess(String exportPath, String paidDate)
            throws IOException, ErrorResponseException {
        Path directPath = generateCoverDirectReport(paidDate);
        String directFileName = directPath.getFileName().toString().split("-")[1];
        onedriveService.saveFile(exportPath, directFileName, directPath);
        directPath.toFile().deleteOnExit();
    }

    public void smartReportProcess(String exportPath, String paidDate) throws IOException, ErrorResponseException {

        String[][] params = { { "SmartPayOut", "0", "1000000" }, { "2SmartPayOut", "1000000", "99999999999" }, };
        for (String[] param : params) {
            Path directPath = generateSmartReport(param[0], paidDate, param[1], param[2]);
            String directFileName = directPath.getFileName().toString().split("-")[1];
            onedriveService.saveFile(exportPath, directFileName, directPath);
            directPath.toFile().deleteOnExit();
        }

    }

    public void coverSmartReportProcess(String exportPath, String paidDate) throws IOException, ErrorResponseException {
        Path directPath = generateCoverSmartReport(paidDate);
        String directFileName = directPath.getFileName().toString().split("-")[1];
        onedriveService.saveFile(exportPath, directFileName, directPath);
        directPath.toFile().deleteOnExit();
    }

    public Integer getHrmsCountGroupSize(String condition, String paidDate) {
        // String sql = " select count( distinct sm_unit_name) from
        // compensation.f_report_hrms(:p_paid_date, :p_condition)";
        Integer groupSize = this.tmpPayAllRepository.getMaxHrmsGroup(Integer.parseInt(paidDate), condition);
        // Object groupSize = this.em.createNativeQuery(sql).setParameter("p_condition",
        // condition)
        // .setParameter("p_paid_date", Integer.parseInt(paidDate)).getSingleResult();

        // log.info("{}, {} groupSize :{} ", condition, paidDate, groupSize);
        return groupSize;// == null ? 0 : Integer.parseInt(groupSize.toString());
    }

    public void hrmsReportProcess(String exportPath, RblSchedule schedule)
            throws IOException, ErrorResponseException, JRException, SQLException {
        HrmsReportType[] hrms = { HrmsReportType.COMPANY, HrmsReportType.DEBT, HrmsReportType.INDIVIDUAL };
        String reportDate = SDF_YYYYMMDDHHMM.format(new Date());
        String periodFromTo = schedule.getComPeriodFrom().getDate() + "-"
                + SDF_DD_MMMM_YYYY.format(schedule.getComPeriodTo());
        String paidDate = SDF_YYYYMMDD.format(schedule.getPaymentDate());

        for (HrmsReportType h : hrms) {
            Path directPath = generateHrmsReport(h, paidDate, periodFromTo, reportDate);
            String directFileName = directPath.getFileName().toString().split("-")[1];
            onedriveService.saveFile(exportPath, directFileName, directPath);
            directPath.toFile().deleteOnExit();
        }

    }

    public void pndReportProcess(String exportPath, String period) throws IOException, ErrorResponseException {
        Path directPath = generatePndReport(period);
        String directFileName = directPath.getFileName().toString().split("-")[1];
        onedriveService.saveFile(exportPath, directFileName, directPath);
        directPath.toFile().deleteOnExit();
    }

    @Modifying
    @Transactional
    public List<TmpPayAll> readDataTmpPayAll(String paidDate, Sheet sheet) {
        List<TmpPayAll> list = new ArrayList<>();

        if (sheet == null) {
            log.info("ReadDataTmpPayAll sheet.getLastRowNum() {}", sheet.getLastRowNum());
            return list;
        }

        log.info("ReadDataTmpPayAll sheet.getLastRowNum() {}", sheet.getLastRowNum());
        for (int i = 5; i <= sheet.getLastRowNum(); i++) {
            Row row = sheet.getRow(i);
            Map<String, Object> r = new HashMap<>();
            r.put("paidDate", paidDate);

            for (int c = 0; c <= 79; c++) {
                Cell cell = row.getCell(c);
                Object o = getCellValue(cell);

                // log.info("[{},{}]", i, c);
                // log.info("[{},{}] : {}", row.getRowNum(), cell.getColumnIndex(), o);
                try {
                    String header = SHEET_PAID_ALL_HEADER[cell.getColumnIndex()];
                    r.put(header, o);
                } catch (ArrayIndexOutOfBoundsException | NullPointerException e) {
                    log.warn("Error at Row[{}] : {} ", row.getRowNum(), o);
                    break;
                }
            }
            // log.info(r);

            TmpPayAll tmp = ObjectUtil.mapper.convertValue(r, TmpPayAll.class);
            list.add(tmp);
        }
        this.tmpPayAllRepository.deleteByPaidDate(new Integer(paidDate));
        this.tmpPayAllRepository.saveAll(list);
        return list;
    }

    @Modifying
    @Transactional
    public List<TmpSummary> readDataTmpSummary(String paidDate, Sheet sheet) {
        List<TmpSummary> list = new ArrayList<>();
        final Set<Integer> ignoreRow = new HashSet<>(Arrays.asList(5, 45, 46, 50, 51, 59));
        if (sheet == null) {
            log.error("Sheet Summary is unavalible");
            return list;
        }

        this.tmpSummaryRepository.deleteByPaidDate(new Integer(paidDate));
        for (int i = 4; i <= 60; i++) {

            if (ignoreRow.contains(i)) {
                continue;
            }
            Row row = sheet.getRow(i);

            Map<String, Object> r = new HashMap<>();
            r.put("paidDate", paidDate);

            for (int c = 0; c <= 6; c++) {
                Cell cell = row.getCell(c);
                Object o = getCellValue(cell);
                // log.info("[{},{}] : {}", row.getRowNum(), cell.getColumnIndex(), o);
                try {
                    String header = SHEET_SUMMARY_HEADER[cell.getColumnIndex()];
                    r.put(header, o);
                } catch (ArrayIndexOutOfBoundsException | NullPointerException e) {
                    log.warn("Error at Row[{}] : {} ", row.getRowNum(), o);
                    break;
                }
            }
            // log.info(r);

            TmpSummary tmp = ObjectUtil.mapper.convertValue(r, TmpSummary.class);
            list.add(tmp);
        }
        this.tmpSummaryRepository.saveAll(list);
        return list;
    }

    @Transactional
    public List<TmpAccount> readDataAccount(String paidDate, Sheet sheet) {
        List<TmpAccount> list = new ArrayList<>();
        if (sheet == null) {
            log.error("Sheet account is unavalible");
            return list;
        }
        Date start = new Date();
        log.info("Start delete database @ {}", start);
        this.tmpAccountRepository.deleteByPaidDate(new Integer(paidDate));
        log.info("Finished delete database, use time {} ms", (System.currentTimeMillis() - start.getTime()) / 1000L);

        for (Row row : sheet) {
            if (row.getRowNum() == 0)
                continue;

            // log.info("Row {}", row.getRowNum());
            Map<String, Object> r = new HashMap<String, Object>();
            r.put("paidDate", paidDate);
            for (Cell cell : row) {
                Object o = getCellValue(cell);
                String header = SHEET_ACCOUNT_HEADER[cell.getColumnIndex()];
                r.put(header, o);
            }
            // log.info(r);
            TmpAccount tmp = ObjectUtil.mapper.convertValue(r, TmpAccount.class);
            list.add(tmp);
        }

        log.info("Start insert to database @ {}", start);
        this.tmpAccountRepository.saveAll(list);
        log.info("Finished insert to database, use time {} ms", (System.currentTimeMillis() - start.getTime()) / 1000L);
        return list;
    }

    public static Object getCellValue(Cell cell) {
        if (cell == null)
            return null;
        switch (cell.getCellType()) {
        case STRING:
            return cell.getStringCellValue();
        case NUMERIC:
            return cell.getNumericCellValue();
        case BOOLEAN:
            return cell.getBooleanCellValue();
        case FORMULA:
            switch (cell.getCachedFormulaResultType()) {
            case BOOLEAN:
                return cell.getBooleanCellValue();
            case NUMERIC:
                return cell.getNumericCellValue();
            case STRING:
                return cell.getStringCellValue();
            default:
                return null;
            }
        default:
            return null;
        }
    }

    // public static void main(String... args) {
    // String[] headers = new String[] { "no", "agent_code", "name", "agent_level",
    // "sm_unit_name", "sm_unit_code",
    // "sm_name", "sd_name", "esd_name", "agent_office", "income_tax",
    // "amount_non_tax",
    // "income_amount_non_tax", "deduct_amount_non_tax", "amount_fund",
    // "income_fund", "deduct_fund", "tax",
    // "net_income", "bank_code", "bank_branch", "bank_name", "bank_account", "fyc",
    // "ryc", "recruiting",
    // "fa_retention", "fa_career", "fa_bonus_m", "fa_bonus_q", "fa_bonus_y",
    // "motivation",
    // "unit_overriding_fy", "unit_overriding_ry", "unit_joining", "unit_fix_allow",
    // "unit_active_fa",
    // "unit_retention", "unit_career", "unit_expand_tier1", "unit_expand_tier2",
    // "unit_expand_extra",
    // "unit_expand_group", "unit_bonus_q", "unit_bonus_y", "group_overriding_fy",
    // "group_overriding_ry",
    // "group_joining", "group_fix_allow", "group_retention", "group_career",
    // "group_expand_tier1",
    // "group_expand_tier2", "group_bonus_q", "group_bonus_y", "line_overriding",
    // "line_fix_allow",
    // "line_expand_tier1", "line_bonus_q", "line_bonus_y", "office_first_year",
    // "office_renewal_year", "debt",
    // "debt_motivation", "debt_fund_com", "debt_fund_unit", "debt_fund_group",
    // "debt_credit_fee",
    // "debt_medical", "debt_other", "debt_shortage_premium", "debt_tranfer_cash",
    // "debt_tranfer_debt",
    // "debt_recive_debt" };
    // List<String> set = new ArrayList<>();
    // for (String str : headers) {
    // String header = ObjectUtil.toCamelCase(str);
    // set.add(header);
    // }

    // System.out.println(set);
    // }
}
