package com.b3.rbl.compensation.service;

import org.springframework.core.io.ByteArrayResource;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;

import com.b3.rbl.compensation.model.EmailSenderModel;
import com.b3.rbl.compensation.model.FileAttachmentModel;

import javax.mail.internet.MimeMessage;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
public class EmailServiceImpl implements EmailService {
    private JavaMailSender mailSender;

    public EmailServiceImpl(JavaMailSender mailSender) throws Exception {
        this.mailSender = mailSender;

    }

    public void sendEmail(String to, String subject, String body) throws Exception {

        MimeMessage message = mailSender.createMimeMessage();
        MimeMessageHelper helper = new MimeMessageHelper(message, true);
        helper.setSubject(subject);
        helper.setTo(to);
        helper.setText(body, true);
        helper.setFrom("salescompensation@rabbitlife.co.th");
        mailSender.send(message);

        log.info("Mail sent to: {}, with subject: {}", to, subject);
    }

    @Override
    public void sendEmail(EmailSenderModel model) throws Exception {

        MimeMessage message = mailSender.createMimeMessage();
        MimeMessageHelper helper = new MimeMessageHelper(message, true);
        helper.setSubject(model.getSubject());
        helper.setTo(model.getTo());
        helper.setText(model.getBody(), true);
        helper.setFrom("salescompensation@rabbitlife.co.th");

        for (FileAttachmentModel file : model.getAttachment()) {
            helper.addAttachment(file.getFilename(), new ByteArrayResource(file.getByteArr()));
        }
        mailSender.send(message);

        log.info("Mail sent to: {}, with subject: {}", String.join(",", model.getTo()), model.getSubject());

    }

}
