package com.b3.rbl.compensation.service;

import com.b3.rbl.compensation.model.jbpm.*;
import com.b3.rbl.compensation.util.ObjectUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;

import lombok.extern.log4j.Log4j2;

import org.apache.commons.math3.analysis.function.Min;
import org.dmg.pmml.time_series.MA;
import org.drools.core.command.runtime.BatchExecutionCommandImpl;
import org.hibernate.transform.Transformers;
import org.kie.api.KieServices;
import org.kie.api.command.BatchExecutionCommand;
import org.kie.api.command.Command;
import org.kie.api.command.KieCommands;
import org.kie.api.runtime.ExecutionResults;
import org.kie.server.api.marshalling.MarshallingFormat;
import org.kie.server.api.model.KieContainerResource;
import org.kie.server.api.model.KieContainerResourceList;
import org.kie.server.api.model.KieServiceResponse;
import org.kie.server.api.model.ServiceResponse;
import org.kie.server.api.model.definition.ProcessDefinition;
import org.kie.server.api.model.instance.ProcessInstance;
import org.kie.server.api.model.instance.ProcessInstanceCustomVars;
import org.kie.server.api.model.instance.ProcessInstanceList;
import org.kie.server.api.model.instance.VariableInstance;
import org.kie.server.client.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.stereotype.Service;


import java.lang.reflect.Array;
import java.sql.Date;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.*;


import javax.persistence.EntityManager;
import javax.sql.DataSource;
import javax.transaction.Transactional;

@Log4j2
@Service
public class JbpmService {
    @Value("${kie.server.container_id}")
    private String CONTAINERID;

    @Value("${kie.server.process_id}")
    private String PROCESSID;

    @Value("${kie.server.url}")
    private String URL;

    @Value("${kie.server.username}")
    private String USERNAME;

    @Value("${kie.server.password}")
    private String PASSWORD;

    private MarshallingFormat FORMAT = MarshallingFormat.JSON;

    private KieServicesConfiguration conf;
    private KieServicesClient kieServicesClient;

    @Autowired
    private EntityManager apiDatasource;

    private List<Map<String, Object>> mapDataToDB = new ArrayList<>(Arrays.asList(new HashMap<String, Object>() {
        {
            put("tax", new HashMap<String, String>() {
                {
                    put("agentCode", "agent_code");
                    put("calculationDate", "trans_date");
                    put("netAmount", "benefit_amount");
                    put("vat", "vat");
                    put("tax", "tax");
                    put("psnInfoId", "psn_info_id");
                    put("scheduleId", "schedule_id");
                    put("isPaid", "is_paid");
                    put("paidDate", "paid_date");
                    put("wht", "wht");
                    put("vatPercent", "vat_percent");
                    put("whtPercent", "wht_percent");
                    put("paidYear", "paid_year");
                    put("paidMonth", "paid_month");
                    put("beforeVat", "before_vat");
                    put("isVat", "isvat");
                    put("isCorporate", "iscorporate");

                }
            });

            put("MonthlyBonus", new HashMap<String, String>() {
                {
                    put("agentCode", "agent_code");
                    put("agentTitle", "agent_title");
                    put("agentName", "agent_Name");
                    put("agentPosition", "agent_level_code_detail");
                    put("benefitCode", "compensation_benefit_code");
                    put("persistencyRate", "persistency_rate");
                    put("persistencyController", "persistency_controller");
                    put("fycAmount", "fyc");
                    put("fycProduction", "fyc_production");
                    put("bonusRate", "compensation_benefit_rate");
                    put("bonus", "compensation_benefit_amount");
                    // put("bonus", "income_tax");
                    // put("bonus", "deduct_tax");
                    

                }
            });
        }
    }));

    public void initialize() {

        conf = KieServicesFactory.newRestConfiguration(URL, USERNAME, PASSWORD, 60000);
        conf.setMarshallingFormat(FORMAT);

        kieServicesClient = KieServicesFactory.newKieServicesClient(conf);

        log.info(kieServicesClient);
    }

    public void disposeAndCreateContainer() {
        // System.out.println("== Disposing and creating containers ==");
        log.info("== Disposing and creating containers ==");

        // Retrieve list of KIE containers
        List<KieContainerResource> kieContainers = kieServicesClient.listContainers().getResult().getContainers();
        if (kieContainers.size() == 0) {
            // System.out.println("No containers available...");
            log.info("No containers available...");
            return;
        }

        // Dispose KIE container
        KieContainerResource container = kieContainers.get(0);
        String containerId = container.getContainerId();
        ServiceResponse<Void> responseDispose = kieServicesClient.disposeContainer(containerId);
        if (responseDispose.getType() == KieServiceResponse.ResponseType.FAILURE) {
            // System.out.println("Error disposing " + containerId + ". Message: ");
            log.info("Error disposing " + containerId + ". Message: ");
            // System.out.println(responseDispose.getMsg());
            log.info(responseDispose.getMsg());
            return;
        }
        // System.out.println("Success Disposing container " + containerId);
        log.info("Success Disposing container " + containerId);
        // System.out.println("Trying to recreate the container...");
        log.info("Trying to recreate the container...");

        // Re-create KIE container
        ServiceResponse<KieContainerResource> createResponse = kieServicesClient.createContainer(CONTAINERID,
                container);
        if (createResponse.getType() == KieServiceResponse.ResponseType.FAILURE) {
            // System.out.println("Error creating " + containerId + ". Message: ");
            log.info("Error creating " + containerId + ". Message: ");
            // System.out.println(responseDispose.getMsg());
            log.info(responseDispose.getMsg());
            return;
        }
        // System.out.println("Container recreated with success!");
        log.info("Container recreated with success!");
    }

    public String listContainers() {
        String containerStr = "";
        KieContainerResourceList containersList = kieServicesClient.listContainers().getResult();
        List<KieContainerResource> kieContainers = containersList.getContainers();
        // System.out.println("Available containers: ");
        log.info("Available containers: ");
        Integer number = 1;
        for (KieContainerResource container : kieContainers) {
            log.info(number);
            number++;
            containerStr += "\t" + container.getContainerId() + " (" + container.getReleaseId() + ")";
            // System.out.println(containerStr);
            log.info(containerStr);
        }
        return containerStr;
    }

    // public IncomeDetails executeCommands(IncomeDetails incomeDetails) {
    // IncomeDetails response = null;
    // System.out.println("== Sending commands to the server ==");
    // RuleServicesClient rulesClient =
    // kieServicesClient.getServicesClient(RuleServicesClient.class);

    // KieCommands commandsFactory = KieServices.get().getCommands();

    // Command<?> insert =
    // commandsFactory.newInsert(incomeDetails,"defaultKieSession");
    // Command<?> fireAllRules = commandsFactory.newFireAllRules();
    // Command<?> batchCommand =
    // commandsFactory.newBatchExecution(Arrays.asList(insert, fireAllRules));

    // // ServiceResponse<String> executeResponse =
    // rulesClient.executeCommands(CONTAINERID, batchCommand );
    // ServiceResponse<ExecutionResults> executeResponse =
    // rulesClient.executeCommandsWithResults(CONTAINERID, batchCommand );

    // if(executeResponse.getType() == KieServiceResponse.ResponseType.SUCCESS) {
    // System.out.println("Commands executed with success! Response: ");
    // response = (IncomeDetails)
    // executeResponse.getResult().getValue("defaultKieSession");
    // System.out.println(executeResponse.getResult());
    // } else {
    // System.out.println("Error executing rules. Message: ");
    // System.out.println(executeResponse.getMsg());
    // }
    // return response;
    // }

    public void listProcesses() {
        // System.out.println("== Listing Business Processes ==");
        log.info("== Listing Business Processes ==");
        QueryServicesClient queryClient = kieServicesClient.getServicesClient(QueryServicesClient.class);
        List<ProcessDefinition> findProcessesByContainerId = queryClient.findProcessesByContainerId(CONTAINERID, 0,
                1000);
        for (ProcessDefinition def : findProcessesByContainerId) {
            // System.out.println(def.getName() + " - " + def.getId() + " v" +
            // def.getVersion());
            log.info(def.getName() + " - " + def.getId() + " v" + def.getVersion());
        }
    }

    public void executeBusinessProcess(AgentDetailData agentDetail) throws JsonProcessingException {
        // System.out.println("== Start Business Processes ==");
        log.info("== Start Business Processes ==");
        ProcessServicesClient processServicesClient = kieServicesClient.getServicesClient(ProcessServicesClient.class);

        Map<String, Object> data = new HashMap<String, Object>();

        Class<?> cls = AgentDetailData.class;
        String classPackage = cls.getPackage().getName().substring(0, 24).concat(cls.getSimpleName());
        // System.out.println("Class : " + classPackage);
        log.info("Class : " + classPackage);
        // for (int i = 0; i < agentDetail.getAgents().size(); i+=(agentDetail.getAgents().size()/4)) {

        //     log.info("i {} end {}",i,Math.min(i+(agentDetail.getAgents().size()/4), agentDetail.getAgents().size()));

        //     Map<String, Object> data = new HashMap<String, Object>();
        //     Map<String, Object> stringMap = new HashMap<String, Object>();
        //     stringMap.put(classPackage, agentDetail.getAgents().subList(i,Math.min(i+(agentDetail.getAgents().size()/4), agentDetail.getAgents().size())));
        //     data.put("data", stringMap);

        //     Long process = processServicesClient.startProcess(CONTAINERID, PROCESSID, data);
            
        // }
        Map<String, Object> stringMap = new HashMap<String, Object>();
        stringMap.put(classPackage, agentDetail);
        data.put("data", stringMap);

        Long process = processServicesClient.startProcess(CONTAINERID, PROCESSID, data);

        // System.out.println("== End Business Processes ==");
        log.info("== End Business Processes ==");

    }


    @Transactional
    public void getDataMonthlyBonus(Date startdate , Date enddate) {

        String sql = " SELECT * from compensation.f_rbl_monthly_bonus_detail(:start_date, :end_date)";
       
        String benefitType = "MonthlyBonus";
        DecimalFormat decimalFormat = new DecimalFormat( "#.###" );

        List<Map<String, Object>> data = new ArrayList();
        List<Map<String, Object>> query = this.apiDatasource.createNativeQuery(sql)
                .setParameter("start_date", startdate)
                .setParameter("end_date", enddate)
                .unwrap(org.hibernate.query.Query.class)
                .setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP).getResultList();
        data = ObjectUtil.columnToCamelCase(query);


    
        log.info("res {}", data.size());
        log.info("resData {}", data);

       Map<String, Map<String, Object>> groupData = new HashMap<>();

       for (Map<String, Object> recorddata : data) {
       
        String agentCode = (String) recorddata.get("agentCode");
       
        Double fyc = ((Number) recorddata.get("fyc")).doubleValue();
        Double fycProduction = ((Number) recorddata.get("fycProduction")).doubleValue();

        String formattedfycProduction = decimalFormat.format(fyc);
        Double afterformatfycProduction = Double.valueOf(formattedfycProduction);

        String formattedfycAmount = decimalFormat.format(fyc);
        Double afterformatfycAmount = Double.valueOf(formattedfycAmount);

        if (groupData.containsKey(agentCode)) {

            Map<String, Object> sumOfFyc = groupData.get(agentCode);
            Double currentFycProduction = (Double) sumOfFyc.get("fycProduction");
            String formattedCurrentProduction = decimalFormat.format(currentFycProduction + afterformatfycProduction);
            sumOfFyc.put("fycProduction", Double.valueOf(formattedCurrentProduction));
            
            Double currentFycAmount = (Double) sumOfFyc.get("fyc");
            String formattedCurrentAmount = decimalFormat.format(currentFycAmount + afterformatfycAmount);
            sumOfFyc.put("fyc", Double.valueOf(formattedCurrentAmount));
        } else {

            Map<String, Object> newRecord = new HashMap<>(recorddata);
            newRecord.put("fycProduction", fycProduction);
            newRecord.put("fyc", fyc);
            groupData.put(agentCode, newRecord);
        }
    }

    
    List<Map<String, Object>> groupDataList = new ArrayList<>(groupData.values());

    log.info("datagroup size {}", groupDataList.size());
    log.info("Datagroup {}", groupDataList);


  
    for (int i = 0; i < groupDataList.size(); i += 30) {
        try {
            log.info("from {} to {}", i, i + 30);
            AgentDetailData agentData = setupPayload(groupDataList.subList(i, Math.min(i + 30, groupDataList.size())), benefitType);
            log.info(agentData.getAgents().size());
            executeBusinessProcess(agentData);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
    }
        
        // for (int i = 0; i < data.size(); i+=30) {
        //     try {
        //         log.info("from {} to {}",i,i+30);
        //         AgentDetailData agentData = setupPayload(data.subList(i, Math.min(i+30, data.size())),benefitType);
        //         log.info(agentData.getAgents().size());
        //         executeBusinessProcess(agentData);
        //     } catch (JsonProcessingException e) {
        //         // TODO Auto-generated catch block
        //         e.printStackTrace();
        //     }
        // }
     
    }


    @Transactional
    public void getDataTax(String scheduleDate) {

        String sql = "SELECT * from compensation.f_rbl_retrieve_data_for_tax_calculation(:pay_ym_t)";
        String benefitType = "Tax";

        List<Map<String, Object>> data = new ArrayList();
        List<Map<String, Object>> query = this.apiDatasource.createNativeQuery(sql)
                .setParameter("pay_ym_t", scheduleDate)
                .unwrap(org.hibernate.query.Query.class)
                .setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP).getResultList();
        data = ObjectUtil.columnToCamelCase(query);

        log.info("res {}", data.size());

        // AgentDetailData agentData = setupPayload(data, benefitType);

        // try {
        //     executeBusinessProcess(agentData);
        // } catch (JsonProcessingException e) {
        //     // TODO Auto-generated catch block
        //     e.printStackTrace();
        // }

        // log.info("agent {}", agentData);

        for (int i = 0; i < data.size(); i+=30) {
            try {
                log.info("from {} to {}",i,i+30);
                AgentDetailData agentData = setupPayload(data.subList(i, Math.min(i+30, data.size())),benefitType);
                log.info(agentData.getAgents().size());
                executeBusinessProcess(agentData);
            } catch (JsonProcessingException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }

    public AgentDetailData setupPayload(List<Map<String, Object>> data, String benefitType) {
        AgentDetailData agentData = new AgentDetailData();
        List<AgentResult> agentResultList = new ArrayList<>();
        int count = 0;
        for (Map<String, Object> map : data) {
            // log.info("{} map {} {}",count, map.entrySet(), map.get("paidDate").toString());
            AgentResult agentResult = new AgentResult();

            if (benefitType.equals("Tax")) {
                agentResult.setAmount(Double.valueOf(String.valueOf(map.get("amount"))));
                agentResult.setAgentCode((String) map.get("agentCode"));
                agentResult.setIsCorporate((Boolean) map.get("isCorporate"));
                agentResult.setIsVat((Boolean) map.get("isVat"));
                // agentResult.setPayYm((String) map.get("payYm"));
                agentResult.setPaidDate((Date) map.get("paidDate"));
                agentResult.setPaidMonth((String) map.get("paidMonth"));
                agentResult.setPaidYear((String) map.get("paidYear"));
                agentResult.setNetAmount(Double.valueOf(String.valueOf(map.get("netAmount"))));
                agentResult.setTaxAccumulate(Double.valueOf(String.valueOf(map.get("taxAccumulate"))));
                agentResult.setBenefitType(benefitType);
                agentResult.setScheduleId((String) map.get("scheduleId"));
                agentResult.setPsnIdInfo((String) map.get("psnInfoId"));
            }

            if (benefitType.equals("MonthlyBonus")) {
                agentResult.setStartDate((Date) map.get("startDate"));
                agentResult.setEndDate((Date) map.get("endDate"));
                agentResult.setAgentCode((String) map.get("agentCode"));
                agentResult.setAgentName((String) map.get("agentName"));
                agentResult.setAgentPosition((String) map.get("agentLevelCodeDetail"));
                agentResult.setPolicyNo((String) map.get("policyNo"));
                agentResult.setFycAmount((Double) map.get("fyc"));
                agentResult.setFycProduction((Double) map.get("fycProduction"));
                // agentResult.setTransactionEffectiveDate((Date) map.get("transactionEffectiveDate"));
                // agentResult.setPolicyStatusCode((String) map.get("policyStatusCode"));
                // agentResult.setPolicyStatusName((String) map.get("policyStatusName"));
                agentResult.setAgentTitle((String) map.get("agentTitle"));
                agentResult.setBenefitCode((String) map.get("aftBenefitCode"));
                agentResult.setPercistencyController(100.0);
                agentResult.setPersistencyRate(100.0);


            }
            
            log.info("obj : {}", agentResult.getBenefitType());
            agentResultList.add(agentResult);
            count++;
        }
        agentData.setAgents(agentResultList);
        // log.info("agents {}", agentResultList.get(0).getBenefitType());
        return agentData;
    }
    @Modifying
    @Transactional
    public void setupData(AgentDetailData data) {
        log.info("data : {}", data.getAgents().get(0).getBenefitType());
        String sql = "INSERT INTO compensation.";
        Integer count = 0;
        for (AgentResult agent : data.getAgents()) {
            if (agent.getBenefitType().equals("Tax")) {
                log.info("paid Date {}", agent.getPaidDate().toString());
                Map<String, String> mp = (Map<String, String>) mapDataToDB.get(0).get("tax");
                if (count == 0) {
                    sql += "rbl_tax_result (" + mp.get("agentCode") + ","
                        + mp.get("calculationDate") + "," + mp.get("netAmount")
                        + "," + mp.get("vat") + ","
                        + mp.get("tax") + "," + mp.get("psnInfoId")
                        + "," + mp.get("scheduleId") + ","
                        + mp.get("isPaid") + "," + mp.get("paidDate")
                        + "," + mp.get("wht") + ","
                        + mp.get("vatPercent") + "," + mp.get("whtPercent")
                        + "," + mp.get("paidYear") + ","
                        + mp.get("paidMonth") + "," + mp.get("beforeVat")
                        + "," + mp.get("isVat") + ","
                        + mp.get("isCorporate") + ") VALUES ";

                        sql += "(" + agent.getAgentCode() + ","
                        + convertDateFormat(agent.getCalculationDate()) + "," + agent.getAmount()
                        + "," + agent.getVat() + ","
                        + agent.getTax() + "," + agent.getPsnIdInfo()
                        + ",'" + agent.getScheduleId() + "',"
                        + agent.getIsPaid() + "," + concatSingleQuote(agent.getPaidDate().toString())
                        + "," + agent.getWht() + ","
                        + agent.getVatPercent() + "," + agent.getWhtPercent()
                        + "," + agent.getPaidYear() + ","
                        + agent.getPaidMonth() + "," + agent.getBeforeVat()
                        + "," + agent.getIsVat() + ","
                        + agent.getIsCorporate() + "),";
                }
                else{
                    sql += "(" + agent.getAgentCode() + ","
                        + convertDateFormat(agent.getCalculationDate()) + "," + agent.getAmount()
                        + "," + agent.getVat() + ","
                        + agent.getTax() + "," + agent.getPsnIdInfo()
                        + ",'" + agent.getScheduleId() + "',"
                        + agent.getIsPaid() + "," + concatSingleQuote(agent.getPaidDate().toString())
                        + "," + agent.getWht() + ","
                        + agent.getVatPercent() + "," + agent.getWhtPercent()
                        + "," + agent.getPaidYear() + ","
                        + agent.getPaidMonth() + "," + agent.getBeforeVat()
                        + "," + agent.getIsVat() + ","
                        + agent.getIsCorporate() + "),";
                }
                

            }
            if (agent.getBenefitType().equals("MonthlyBonus")) {

                log.info("start Date {}", agent.getStartDate().toString());
                log.info("end Date {}", agent.getEndDate().toString());
                Map<String, String> mp = (Map<String, String>) mapDataToDB.get(0).get("MonthlyBonus");
                if (count == 0) {
                    sql += "rbl_benefit_result (" 
                        + mp.get("agentCode") + ","
                        + mp.get("agentTitle") + "," 
                        + mp.get("agentName") + "," 
                        + mp.get("agentPosition") + ","
                        + mp.get("benefitCode") + "," 
                        + mp.get("persistencyRate") + "," 
                        + mp.get("persistencyController") + ","
                        + mp.get("fycAmount") + "," 
                        + mp.get("fycProduction") + "," 
                        + mp.get("bonusRate") + ","
                        + mp.get("bonus") + ","
            
                        + ") VALUES ";

                        sql += "(" 
                        + agent.getAgentCode() + ","
                        + agent.getAgentTitle() + ","
                        + agent.getAgentName() + "," 
                        + agent.getAgentPosition() + ","
                        + agent.getBenefitCode() + "," 
                        + agent.getPersistencyRate() + ",'" 
                        + agent.getPercistencyController() + "',"
                        + agent.getFycAmount() + "," 
                        + agent.getFycProduction() + "," 
                        + agent.getBonusRate() + "," 
                        + agent.getBonus() + "," 
     
                        // + concatSingleQuote(agent.getStartDate().toString()) + "," 
                        // + concatSingleQuote(agent.getEndDate().toString()) + "," 
                        + "),";
                }
                else{
                    sql += "(" 
                    + agent.getAgentCode() + ","
                    + agent.getAgentTitle() + ","
                    + agent.getAgentName() + "," 
                    + agent.getAgentPosition() + ","
                    + agent.getBenefitCode() + "," 
                    + agent.getPersistencyRate() + ",'" 
                    + agent.getPercistencyController() + "',"
                    + agent.getFycAmount() + "," 
                    + agent.getFycProduction() + "," 
                    + agent.getBonusRate() + "," 
                    + agent.getBonus() + "," 
                    + "),";
                }
                       
            }
            count++;
        }
        sql = sql.substring(0,sql.length()-1);
        log.info("sql {}",sql );
        saveData(sql);

    }

    @Modifying
    @Transactional
    public void saveData(String sql){
        try {
            this.apiDatasource.createNativeQuery(sql).executeUpdate();
        } catch (Exception e) {
            log.info("err {} {}",e.getMessage(),e);
            // TODO: handle exception
        }
    }
    public String convertDateFormat(String strDate){
          // Format of the original date string
        SimpleDateFormat originalFormat = new SimpleDateFormat("dd/MM/yyyy");

        // Desired format
        SimpleDateFormat targetFormat = new SimpleDateFormat("yyyy-MM-dd");

        String formattedDate = "";

        try {
            // Parse the original date string to a Date object
            java.util.Date date = originalFormat.parse(strDate);

            // Format the Date object to the desired format
            formattedDate = targetFormat.format(date);

            // Print the formatted date
           log.info(formattedDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return "'"+formattedDate+"'";
    }

    public String concatSingleQuote(String str){
        return "'"+str+"'";
    }

    // public AgentResult calculationFyc(AgentDetails agents) throws
    // JsonProcessingException {
    // AgentResult agent = new AgentResult();
    // List<AgentDetail> details = new ArrayList<AgentDetail>();
    // Double amount = 0.00;
    // for (AgentDetail agentDetail : agents.agents){
    // AgentDetail detail = new AgentDetail();
    // detail.setFycAmount(agentDetail.getFycAmount());
    // detail.setAgentCode(agentDetail.getAgentCode());
    // detail.setPolicyNo(agentDetail.getPolicyNo());
    // detail.setAgentPosition(agentDetail.getAgentPosition());
    //
    // details.add(detail);
    //
    // amount += agentDetail.getFycAmount();
    // agent.setAgentCode(agentDetail.getAgentCode());
    // agent.setAgentPosition(agentDetail.getAgentPosition());
    // agent.setFycAmount(amount);
    // }
    // agent.setAgentDetail(details);
    //
    // executeBusinessProcess(agent);
    //
    // return agent;
    // }

    public String getHelloWorld() {
        return "Hello World";
    }
}
