package com.b3.rbl.compensation.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;

import org.hibernate.transform.Transformers;
import org.springframework.data.util.Streamable;
import org.springframework.stereotype.Service;

import com.b3.rbl.compensation.db.entity.MetaType;
import com.b3.rbl.compensation.db.entity.RblAgentReport;
import com.b3.rbl.compensation.db.entity.RblSchedule;
import com.b3.rbl.compensation.db.repository.MetaTypeRepository;
import com.b3.rbl.compensation.db.repository.RblAgenReportRepository;
import com.b3.rbl.compensation.db.repository.RblScheduleRepository;
import com.b3.rbl.compensation.util.ObjectUtil;

import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
public class ReportService {
    private RblAgenReportRepository agenReportRepository;
    private RblScheduleRepository scheduleRepository;
    private MetaTypeRepository metaTypeRepository;
    private EntityManager em;

    public ReportService(RblAgenReportRepository agenReportRepository, RblScheduleRepository scheduleRepository,
            MetaTypeRepository metaTypeRepository, EntityManager em) {
        this.agenReportRepository = agenReportRepository;
        this.scheduleRepository = scheduleRepository;
        this.metaTypeRepository = metaTypeRepository;
        this.em = em;
    }

    public List<RblSchedule> getScheduleMaster() {
        Iterable<RblSchedule> it = scheduleRepository.findAllByOrderByPaymentDateAsc();
        return Streamable.of(it).toList();
    }

    public List<Map<String, Object>> searchReport(String reportName, String periodId, String agentCode,
            String agentCodeFilter) {
        log.info("reportName {}", reportName);
        log.info("periodId {}", periodId);
        log.info("agentCode {}", agentCode);

        final String sql = " select * from compensation.f_search_agent_report(:reportName, :periodId, :agentCode) where agent_code = :agentCodeFilter or COALESCE(:agentCodeFilter, '') = '' ";
        List<Map<String, Object>> data = new ArrayList();
        List<Map<String, Object>> query = this.em.createNativeQuery(sql).setParameter("reportName", reportName)
                .setParameter("periodId", periodId).setParameter("agentCode", agentCode)
                .setParameter("agentCodeFilter", agentCodeFilter).unwrap(org.hibernate.query.Query.class)
                .setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP).getResultList();
        data = ObjectUtil.columnToCamelCase(query);

        return data;
    }

    public List<MetaType> getReportMaster() {
        Iterable<MetaType> it = metaTypeRepository.findByGroupTypeAndIsDelete("R01", false);
        return Streamable.of(it).toList();
    }

}
