package com.b3.rbl.onedrive.container.items;

import com.b3.rbl.onedrive.client.Client;
import com.b3.rbl.onedrive.container.facet.RemoteItemFacet;
import com.b3.rbl.onedrive.container.facet.SearchResultFacet;
import com.b3.rbl.onedrive.container.facet.SharePointIdsFacet;
import com.b3.rbl.onedrive.container.facet.SharedFacet;
import com.b3.rbl.onedrive.container.items.pointer.IdPointer;
import com.b3.rbl.onedrive.exceptions.ErrorResponseException;
import lombok.Getter;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.net.URI;

public abstract class AbstractRemoteItem extends AbstractDriveItem implements RemoteItem {
	@Getter(onMethod = @__(@Override)) protected @NotNull RemoteItemFacet remoteItem;
	@Getter(onMethod = @__(@Override)) protected @NotNull IdPointer remotePointer;

	AbstractRemoteItem(@NotNull String id, @NotNull String createdDateTime, @Nullable String description,
					   @NotNull String eTag, @NotNull String lastModifiedDateTime, @NotNull String name,
					   @NotNull URI webUrl, @NotNull Client client, @NotNull String cTag, @Nullable String deleted,
					   @NotNull ItemReference parentReference, @Nullable SearchResultFacet searchResult,
					   @Nullable SharedFacet shared, @Nullable SharePointIdsFacet sharePointIds, URI webDavUrl,
					   @NotNull RemoteItemFacet remoteItem) {
		super(id, remoteItem.getCreator(), createdDateTime, description, eTag, remoteItem.getLastModifier(),
				lastModifiedDateTime, name, webUrl, client, cTag, deleted, remoteItem.getFileSystemInfo(),
				parentReference, searchResult, shared, sharePointIds, remoteItem.getSize(), webDavUrl);
		this.remoteItem = remoteItem;

		createPointers();
	}

	@Override
	protected void createPointers() {
		if (parentReference.pathPointer != null) {
			assert name != null : "name is null";
			this.pathPointer = parentReference.pathPointer.resolve(name);
		}
		this.idPointer = new IdPointer(id, parentReference.driveId);
		this.remotePointer = new IdPointer(remoteItem.getId(), remoteItem.getParentReference().driveId);
	}

	public @NotNull String getRemoteDriveID() {return remoteItem.getParentReference().driveId;}

	public @NotNull String getRemoteID() {return remoteItem.getId();}


	@Override public @NotNull DriveItem fetchRemoteItem() throws ErrorResponseException {
		return client.getItem(remotePointer);
	}
}
