package com.b3.rbl.onedrive.container.items;

import com.b3.rbl.onedrive.client.Client;
import com.b3.rbl.onedrive.container.facet.*;
import com.b3.rbl.onedrive.network.async.DownloadFuture;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.net.URI;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * @author <a href="mailto:bh322yoo@gmail.com" target="_top">isac322</a>
 */
public class RemoteFileItem extends AbstractRemoteItem implements FileItem {
	RemoteFileItem(@NotNull String id, @NotNull String createdDateTime, @Nullable String description,
				   @NotNull String eTag, @NotNull String lastModifiedDateTime, @NotNull String name,
				   @NotNull URI webUrl, @NotNull Client client, @NotNull String cTag, @Nullable String deleted,
				   @NotNull ItemReference parentReference, @Nullable SearchResultFacet searchResult,
				   @Nullable SharedFacet shared, @Nullable SharePointIdsFacet sharePointIds, URI webDavUrl,
				   @NotNull RemoteItemFacet remoteItem) {
		super(id, createdDateTime, description, eTag, lastModifiedDateTime, name, webUrl, client, cTag, deleted,
				parentReference, searchResult, shared, sharePointIds, webDavUrl, remoteItem);
	}

	@Override
	public void download(@NotNull String path) throws IOException {
		assert this.name != null : "this.name is null";
		client.download(idPointer, Paths.get(path), this.name);
	}

	@Override
	public void download(@NotNull String path, @NotNull String newName) throws IOException {
		client.download(idPointer, Paths.get(path), newName);
	}

	@Override
	public void download(@NotNull Path folderPath) throws IOException {
		assert this.name != null : "this.name is null";
		client.download(idPointer, folderPath, this.name);
	}

	// TODO: handling overwriting file

	@Override
	public void download(@NotNull Path folderPath, String newName) throws IOException {
		client.download(idPointer, folderPath, newName);
	}


	@Override
	public @NotNull DownloadFuture downloadAsync(@NotNull Path folderPath) throws IOException {
		return client.downloadAsync(idPointer, folderPath, this.name);
	}

	@Override
	public @NotNull DownloadFuture downloadAsync(@NotNull Path folderPath, String newName) throws IOException {
		return client.downloadAsync(idPointer, folderPath, newName);
	}


	@Override public @Nullable String getMimeType() {
		assert remoteItem.getFile() != null : "remoteItem.getFile() is null";
		return remoteItem.getFile().getMimeType();
	}

	@Override public @Nullable String getCRC32() {
		assert remoteItem.getFile() != null : "remoteItem.getFile() is null";
		return remoteItem.getFile().getCrc32Hash();
	}

	@Override public @Nullable String getSHA1() {
		assert remoteItem.getFile() != null : "remoteItem.getFile() is null";
		return remoteItem.getFile().getSha1Hash();
	}

	@Override public @Nullable String getQuickXorHash() {
		assert remoteItem.getFile() != null : "remoteItem.getFile() is null";
		return remoteItem.getFile().getQuickXorHash();
	}

	@Override public @Nullable AudioFacet getAudio() {return null;}

	@Override public @Nullable ImageFacet getImage() {return null;}

	@Override public @Nullable LocationFacet getLocation() {return null;}

	@Override public @Nullable PhotoFacet getPhoto() {return null;}

	@Override public @Nullable VideoFacet getVideo() {return null;}
}
